\\ Copyright 2015, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  If not, see <http://www.gnu.org/licenses/>.


read("c-curve-defines.gp");

check_count = 0;
check_exact(got,want, name="") =
{
  check_count++;
  if(got!= want,
     print("\nwrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count));
}

xy_rotate_minus45(v) = [ v[1]+v[2], -v[1]+v[2] ]/sqrt2;
xy_rotate_minus45_times(v,n) = for(i=1,n%8, v=xy_rotate_minus45(v)); v;

b_real_sign_vector = [1,1,0,-1, -1,-1,0,1];
b_real_sign(k) = b_real_sign_vector[(k%8)+1];
b_real(k) = b_real_sign(k) * 2^floor(k/2);
b_imag_sign_vector = [0,1,1,1, 0,-1,-1,-1];
b_imag_sign(k) = b_imag_sign_vector[(k%8)+1];
b_imag(k) = b_imag_sign(k) * 2^floor(k/2);
check_exact(vector(64,k,my(k=k-1); real(b^k)), \
            vector(64,k,my(k=k-1);  b_real(k)));
check_exact(vector(64,k,my(k=k-1); imag(b^k)), \
            vector(64,k,my(k=k-1);  b_imag(k)));

RS(j) = 2^j;
RZ(j) = 2*(2^j-1);

\\-----------------------------------------------------------------------------

Rtop_samples = [0, 1/2*sqrt2, 1, (1+1/2)*sqrt2, 3, (3+1/2)*sqrt2, 7 ];

Rtop_by_midpoints(k) = \
  vecmax(apply(imag,c_curve_midpoints(k)*conj(b)^(k-4))) / sqrt2^(k-4) \
  + if(k%2,sqrt2/4,0);
Rtop_by_midpoints = memoize(Rtop_by_midpoints);
check_exact(vector(#Rtop_samples,k,my(k=k-1); Rtop_by_midpoints(k)), \
            Rtop_samples, "Rtop_by_midpoints() samples")

Rtop(k) = if(k==0,0, (Rtop(k-1) + if(k%2==1,1/2))*sqrt2);
check_exact(vector(#Rtop_samples,k,my(k=k-1); Rtop(k)), Rtop_samples, \
            "Rtop() samples")

check_exact(vector(14,k, Rtop(k-1)), vector(14,k, Rtop_by_midpoints(k-1)))

Rtop(k) = if(k%2==0, 2^(k/2)-1, (2^((k-1)/2) - 1/2)*sqrt2 );
check_exact(vector(14,k, Rtop(k-1)), vector(14,k, Rtop_by_midpoints(k-1)))

Rtop(k) = sqrt2^k - if(k%2==0, 1, 1/2*sqrt2);
check_exact(vector(14,k, Rtop(k-1)), vector(14,k, Rtop_by_midpoints(k-1)))


\\-----------------------------------------------------------------------------

GRtotal_by_midpoints(k) = \
  imag(vecsum(c_curve_R_midpoints(k))*conj(b)^(k-4)) / sqrt2^(k-4);
GRtotal_by_midpoints = memoize(GRtotal_by_midpoints);

GR_by_midpoints(k) = GRtotal_by_midpoints(k) / R(k);

vector(12,k,my(k=k-1); printtex(GRtotal_by_midpoints(k)))

GRtotaltop_xy(k) = \
  my(x=-1/2*sqrt2^k, y=Rtop(k)); \
  if(k%2==0, [x, y        ] * RS(k/2), \
             [x, y-sqrt2/4] * RZ(k\2));
GRtotal(k) = my(v=[0,0]); \
  for(j=0,k-1, \
      v += 2*xy_rotate_minus45_times(GRtotaltop_xy(j), k-j)); \
  v += GRtotaltop_xy(k); \
  v[2];

check_exact(vector(14,k, GRtotal(k-1)), vector(14,k, GRtotal_by_midpoints(k-1)))


GRtotaltop(k) = \
  my(z = -1/2*sqrt(2)^k + Rtop(k)*1.0*I); \
  if(k%2==0, z * RS(k/2),  (z - sqrt(2)/4*I) * RZ(k\2));
GRtotal(k) = my(z=0); \
  for(j=0,k-1, \
      z += 2 * (conj(b)/sqrt(2))^(k-j) * GRtotaltop(j)); \
  z += GRtotaltop(k); \
  imag(z);

check_exact(vector(12,k, nearly_equal(GRtotal(k-1),GRtotal_by_midpoints(k-1))), \
            vector(12,k, 1))


GRtotaltop(k) = \
  my(z = -1/2*sqrt(2)^k + (sqrt(2)^k - if(k%2==0, 1, 1/2*sqrt(2)))*I); \
  if(k%2==0, z * sqrt(2)^k, \
             (z - sqrt(2)/4*I) * (sqrt(2)*sqrt(2)^k - 2));
GRtotal(k) = my(z=0); \
  for(j=0,k-1, \
      z += 2 * (conj(b)/sqrt(2))^(k-j) * GRtotaltop(j)); \
  z += GRtotaltop(k); \
  imag(z);

check_exact(vector(12,k, nearly_equal(GRtotal(k-1),GRtotal_by_midpoints(k-1))), \
            vector(12,k, 1))

GRtotaltop(k) = \
  my(z = (-1/2+I)*sqrt(2)^k - if(k%2==0, I, 1/2*sqrt(2)*I)); \
  if(k%2==0, \
     (-1/2+I)*2^k - sqrt(2)^k*I, \
     \
     (-1/2+I)*sqrt(2) * 2^k \
     + (1-2*I)        * sqrt(2)^k \
     - 3/4*sqrt(2)*sqrt(2)*I * sqrt(2)^k \
     + 3/2*sqrt(2)*I);
\\
GRtotal(k) = my(z=0); \
  for(j=0,k-1, \
      z += 2 * (conj(b)/sqrt(2))^(k-j) * GRtotaltop(j)); \
  z += GRtotaltop(k); \
  imag(z);

check_exact(vector(12,k, nearly_equal(GRtotal(k-1),GRtotal_by_midpoints(k-1))), \
            vector(12,k, 1))

GRtotaltop(k) = \
  if(k%2==0, \
     (-1/2+I) * 2^k \
     - I      * sqrt(2)^k, \
     \
     (-1/2+I)*sqrt(2) * 2^k \
     + (1 - 7/2*I)    * sqrt(2)^k \
     + 3/2*sqrt(2)*I  );
\\
GRtotal(k) = my(z=0); \
  for(j=0,k-1, \
      z += 2 * (conj(b)/sqrt(2))^(k-j) * GRtotaltop(j)); \
  z += GRtotaltop(k); \
  imag(z);

check_exact(vector(12,k, nearly_equal(GRtotal(k-1),GRtotal_by_midpoints(k-1))), \
            vector(12,k, 1))


GRtotaltop(k) = \
  if(k%2==0, \
     (-1/2+I) * 2^k \
     - I      * sqrt(2)^k, \
     \
     (-1/2+I)*sqrt(2) * 2^k \
     + (1 - 7/2*I)    * sqrt(2)^k \
     + 3/2*sqrt(2)*I  );
GRtotaltop_sum(k) =  sum(j=0,k-1, (conj(b)/sqrt(2))^(k-j) * GRtotaltop(j));
GRtotal(k) = imag( 2*GRtotaltop_sum(k) + GRtotaltop(k) );

check_exact(vector(12,k, nearly_equal(GRtotal(k-1),GRtotal_by_midpoints(k-1))), \
            vector(12,k, 1))

GRtotaltop_sum(k) =
{
  sum(j=0,k-1,
      if(j%2==0,  (conj(b)/sqrt(2))^(k-j) * (-1/2+I) * 2^j, 0 )
      + if(j%2==0,  (conj(b)/sqrt(2))^(k-j) * - I * sqrt(2)^j, 0 )

      + if(j%2==0,  0, (conj(b)/sqrt(2))^(k-j) * (-1/2+I)*sqrt(2) * 2^j )
      + if(j%2==0,  0, (conj(b)/sqrt(2))^(k-j) * (1 - 7/2*I)    * sqrt(2)^j )
      + if(j%2==0,  0, (conj(b)/sqrt(2))^(k-j) * 3/2*sqrt(2)*I )
     );
}
GRtotal(k) = imag( 2*GRtotaltop_sum(k) + GRtotaltop(k) );

check_exact(vector(12,k, nearly_equal(GRtotal(k-1),GRtotal_by_midpoints(k-1))), \
            vector(12,k, 1))

GRtotaltop_sum(k) =
{
  (conj(b)/sqrt(2))^k
  * sum(j=0,k-1,
      (-1/2+I) * if(j%2==0,  (2*sqrt(2)/conj(b))^j, 0 )
      - I      * if(j%2==0,  (2/conj(b))^j,         0 )

      + (-1/2+I)*sqrt(2) * if(j%2==0,  0, (2*sqrt(2)/conj(b))^j )
      + (1 - 7/2*I)      * if(j%2==0,  0, (2/conj(b))^j )
      + 3/2*sqrt(2)*I    * if(j%2==0,  0, (sqrt(2)/conj(b))^j )
     );
}
GRtotal(k) = imag( 2*GRtotaltop_sum(k) + GRtotaltop(k) );

check_exact(vector(12,k, nearly_equal(GRtotal(k-1),GRtotal_by_midpoints(k-1))), \
            vector(12,k, 1))

GRtotaltop_sum(k) =
{
  (conj(b)/sqrt(2))^k
  * (
      (9/34 + 1/17*I) * ((4*I)^(floor((k-1)/2) + 1) - 1)
      + (-2/5 + 1/5*I) * ((2*I)^(floor((k-1)/2) + 1) - 1)

     + sum(j=0,k-2,
       (-3 + I)        * if(j%2==0, (2*sqrt(2)/conj(b))^j,  0 )
      + (9/2 - 5/2*I)  * if(j%2==0, (2/conj(b))^j, 0 )
      + (-3/2 + 3/2*I) * if(j%2==0, (sqrt(2)/conj(b))^j, 0 )
     )
   );
}
GRtotal(k) = imag( 2*GRtotaltop_sum(k) + GRtotaltop(k) );

check_exact(vector(12,k, nearly_equal(GRtotal(k-1),GRtotal_by_midpoints(k-1))), \
            vector(12,k, 1))

GRtotaltop_sum(k) =
{
  (conj(b)/sqrt(2))^k
  * (
      (9/34 + 1/17*I) * ((4*I)^(floor((k-1)/2) + 1) - 1)
      + (-2/5 + 1/5*I) * ((2*I)^(floor((k-1)/2) + 1) - 1)

     + sum(h=0,floor((k-2)/2),
       (-3 + I)        * ((2*sqrt(2)/conj(b))^2)^h
      + (9/2 - 5/2*I)  * ((2/conj(b))^2)^h
      + (-3/2 + 3/2*I) * ((sqrt(2)/conj(b))^2)^h
     )
   );
}
GRtotal(k) = imag( 2*GRtotaltop_sum(k) + GRtotaltop(k) );

check_exact(vector(12,k, nearly_equal(GRtotal(k-1),GRtotal_by_midpoints(k-1))), \
            vector(12,k, 1))

GRtotaltop_sum(k) =
{
  (conj(b)/sqrt(2))^k
  * (
      (9/34 + 1/17*I) * ((4*I)^(floor((k-1)/2) + 1) - 1)
      + (-2/5 + 1/5*I) * ((2*I)^(floor((k-1)/2) + 1) - 1)

     + sum(h=0,floor((k-2)/2),
       (-3 + I)        * (4*I)^h
      + (9/2 - 5/2*I)  * (2*I)^h
      + (-3/2 + 3/2*I) * I^h
     )
   );
}
GRtotal(k) = imag( 2*GRtotaltop_sum(k) + GRtotaltop(k) );

check_exact(vector(12,k, nearly_equal(GRtotal(k-1),GRtotal_by_midpoints(k-1))), \
            vector(12,k, 1))


GRtotaltop_sum(k) =
{
  (conj(b)/sqrt(2))^k
  * (
      (9/34 + 1/17*I) * ((4*I)^(floor((k-1)/2) + 1) - 1)
      + (-2/5 + 1/5*I) * ((2*I)^(floor((k-1)/2) + 1) - 1)

      + (-3 + I)        * ( (4*I)^(floor((k-2)/2) + 1) - 1 ) / (4*I - 1)
      + (9/2 - 5/2*I)  * ((2*I)^(floor((k-2)/2) + 1) - 1) / (2*I - 1)
      + (-3/2 + 3/2*I) * (I^(floor((k-2)/2) + 1) - 1) / (I-1)
     );
}
GRtotal(k) = imag( 2*GRtotaltop_sum(k) + GRtotaltop(k) );

check_exact(vector(12,k, nearly_equal(GRtotal(k-1),GRtotal_by_midpoints(k-1))), \
            vector(12,k, 1))


GRtotaltop_sum(k) =
{
  (conj(b)/sqrt(2))^k
  * (
      (9/34 + 1/17*I) * (4*I)^(floor((k-1)/2) + 1)
      + (-2/5 + 1/5*I) * (2*I)^(floor((k-1)/2) + 1)

      + (7/17 + 11/17*I)    * (4*I)^(floor((k-2)/2) + 1)
      + (-19/10 - 13/10*I)  * (2*I)^(floor((k-2)/2) + 1)
      +  3/2 * I^floor(k/2)
      - 3/2
      - (-19/10 - 13/10*I)
      - (7/17 + 11/17*I) - (9/34 + 1/17*I) - (-2/5 + 1/5*I)
     );
}
GRtotal(k) = imag( 2*GRtotaltop_sum(k) + GRtotaltop(k) );

check_exact(vector(12,k, nearly_equal(GRtotal(k-1),GRtotal_by_midpoints(k-1))), \
            vector(12,k, 1))

GRtotaltop_sum(k) =
{
  (conj(b)/sqrt(2))^k
  * (
        (9/34 + 1/17*I) * (4*I)^(floor((k)/2)) * vector_modulo([1,4*I],k)
      + (7/17 + 11/17*I)    * (4*I)^(floor(k/2))

      + (-2/5 + 1/5*I)  * (2*I)^(floor((k)/2)) * vector_modulo([1,2*I],k)
      + (-19/10 - 13/10*I)  * (2*I)^(floor(k/2))

      +  3/2 * I^floor(k/2)
      + 21/170 + 67/170*I
     );
}
GRtotal(k) = imag( 2*GRtotaltop_sum(k) + GRtotaltop(k) );

check_exact(vector(12,k, nearly_equal(GRtotal(k-1),GRtotal_by_midpoints(k-1))), \
            vector(12,k, 1))

GRtotaltop_sum(k) =
{
  (conj(b)/sqrt(2))^k
  * (
     (sqrt(2)*b)^k
        * vector_modulo([23/34 + 12/17*I,
                         (3/17 + 29/17*I)/(2*b/sqrt(2)) ],k)

      + b^k
        * vector_modulo([-23/10 - 11/10*I,
                         (-23/10 - 21/10*I) * conj((sqrt(2)*b/sqrt(2)))/2], k)
     )
  + vector_modulo([3/2, 3/4*sqrt(2) - 3/4*sqrt(2)*I ], k)
  + (conj(b)/sqrt(2))^k * (21/170 + 67/170*I)
  ;
}
GRtotal(k) = imag( 2*GRtotaltop_sum(k) + GRtotaltop(k) );

check_exact(vector(12,k, nearly_equal(GRtotal(k-1),GRtotal_by_midpoints(k-1))), \
            vector(12,k, 1))

GRtotaltop_sum(k) =
{
  2^k * vector_modulo([23/34 + 12/17*I, (8/17 + 13/34*I)*sqrt(2) ], k)
  + sqrt(2)^k * vector_modulo([-23/10 - 11/10*I, -22/10 + 1/10*I ], k)
  + (conj(b)/sqrt(2))^k * (21/170 + 67/170*I)
  + vector_modulo([3/2, 3/2*(conj(b)/sqrt(2)) ], k)
  ;
}
GRtotal(k) = imag( 2*GRtotaltop_sum(k) + GRtotaltop(k) );

check_exact(vector(12,k, nearly_equal(GRtotal(k-1),GRtotal_by_midpoints(k-1))), \
            vector(12,k, 1))

GRtotaltop_imag(k) = GRtotaltop_xy(k)[2];
GRtotaltop_sum_imag(k) =
{
imag( 
  2^k * vector_modulo([23/34 + 12/17*I, (8/17 + 13/34*I)*sqrt(2) ], k)
  + sqrt(2)^k * vector_modulo([-23/10 - 11/10*I, -22/10 + 1/10*I ], k)
  + (conj(b)/sqrt(2))^k * (21/170 + 67/170*I)
  + vector_modulo([3/2, 3/2*(conj(b)/sqrt(2)) ], k)
  );
}
GRtotal(k) = 2*GRtotaltop_sum_imag(k) + GRtotaltop_imag(k);

check_exact(vector(12,k, nearly_equal(GRtotal(k-1),GRtotal_by_midpoints(k-1))), \
            vector(12,k, 1))

GRtotaltop_imag(k) =
{
  if(k%2==0, Rtop(k)             * RS(k/2),
             (Rtop(k) - sqrt2/4) * RZ(k\2));
}
GRtotaltop_sum_imag(k) =
{
imag( 
  2^k * vector_modulo([23/34 + 12/17*I, (8/17 + 13/34*I)*sqrt(2) ], k)
  + sqrt(2)^k * vector_modulo([-23/10 - 11/10*I, -22/10 + 1/10*I ], k)
  + (conj(b)/sqrt(2))^k * (21/170 + 67/170*I)
  + vector_modulo([3/2, 3/2*(conj(b)/sqrt(2)) ], k)
  );
}
GRtotal(k) = 2*GRtotaltop_sum_imag(k) + GRtotaltop_imag(k);

check_exact(vector(12,k, nearly_equal(GRtotal(k-1),GRtotal_by_midpoints(k-1))), \
            vector(12,k, 1))

GRtotaltop_imag(k) =
{
  my(rtop = sqrt(2)^k - if(k%2==0, 1, 1/2*sqrt(2)));
  if(k%2==0, rtop             * RS(k/2),
             (rtop - sqrt2/4) * RZ(k\2));
}
GRtotaltop_sum_imag(k) =
{
imag( 
  2^k * vector_modulo([23/34 + 12/17*I, (8/17 + 13/34*I)*sqrt(2) ], k)
  + sqrt(2)^k * vector_modulo([-23/10 - 11/10*I, -22/10 + 1/10*I ], k)
  + (conj(b)/sqrt(2))^k * (21/170 + 67/170*I)
  + vector_modulo([3/2, 3/2*(conj(b)/sqrt(2)) ], k)
  );
}
GRtotal(k) = 2*GRtotaltop_sum_imag(k) + GRtotaltop_imag(k);

check_exact(vector(12,k, nearly_equal(GRtotal(k-1),GRtotal_by_midpoints(k-1))), \
            vector(12,k, 1))

GRtotal(k) =
{
  my(rtop = sqrt(2)^k - if(k%2==0, 1, 1/2*sqrt(2)));
  2*imag( 
  2^k * vector_modulo([23/34 + 12/17*I, (8/17 + 13/34*I)*sqrt(2) ], k)
  + sqrt(2)^k * vector_modulo([-23/10 - 11/10*I, -22/10 + 1/10*I ], k)
  + (conj(b)/sqrt(2))^k * (21/170 + 67/170*I)
  + vector_modulo([3/2, 3/2*(conj(b)/sqrt(2)) ], k)
  )
  + if(k%2==0, rtop             * RS(k/2),
               (rtop - sqrt2/4) * RZ(k\2));
}
check_exact(vector(12,k, nearly_equal(GRtotal(k-1),GRtotal_by_midpoints(k-1))), \
            vector(12,k, 1))

GRtotal(k) =
{
  imag( 
  + (conj(b)/sqrt(2))^k * (21/85 + 67/85*I)
  )
  + if(k%2==0, (sqrt(2)^k - if(k%2==0, 1, 1/2*sqrt(2)))   * RS(k/2),
               ((sqrt(2)^k - if(k%2==0, 1, 1/2*sqrt(2))) - sqrt2/4) * RZ(k\2))
  + 2^k * vector_modulo([24/17, 13/17*sqrt(2) ], k)
  + sqrt(2)^k * vector_modulo([ - 11/5,  1/5], k)
  + vector_modulo([0, -3/sqrt(2) ], k)
  ;
}
check_exact(vector(12,k, nearly_equal(GRtotal(k-1),GRtotal_by_midpoints(k-1))), \
            vector(12,k, 1))


GRtotal(k) =
{
  imag( 
  + (conj(b)/sqrt(2))^k * (21/85 + 67/85*I)
  )
  + if(k%2==0, 0,
               (sqrt(2)^k - 1/2*sqrt(2) - sqrt(2)/4) * RZ(k\2))
  + 2^k * vector_modulo([41/17, 13/17*sqrt(2) ], k)
  + sqrt(2)^k * vector_modulo([-16/5, 1/5], k)
  + vector_modulo([0, -3/sqrt(2) ], k)
  ;
}
check_exact(vector(12,k, nearly_equal(GRtotal(k-1),GRtotal_by_midpoints(k-1))), \
            vector(12,k, 1))


GRtotal(k) =
{
  imag( 
  + (conj(b)/sqrt(2))^k * (21/85 + 67/85*I)
  )
  + if(k%2==0, 0,
               (sqrt(2)^k - 3/4*sqrt(2)) * (sqrt(2)^k * sqrt(2) - 2))
  + 2^k * vector_modulo([41/17, 13/17*sqrt(2) ], k)
  + sqrt(2)^k * vector_modulo([-16/5, 1/5], k)
  + vector_modulo([0, -3/sqrt(2) ], k)
  ;
}
check_exact(vector(12,k, nearly_equal(GRtotal(k-1),GRtotal_by_midpoints(k-1))), \
            vector(12,k, 1))

GRtotal(k) =
{
  imag( 
  + (conj(b)/sqrt(2))^k * (21/85 + 67/85*I)
  )
  + if(k%2==0, 0,
               (sqrt(2)^k) * (- 2)
             + (sqrt(2)^k) * (sqrt(2)^k * sqrt(2))
             + ( - 3/4*sqrt(2)) * (- 2)
             + ( - 3/4*sqrt(2)) * (sqrt(2)^k * sqrt(2)))
  + 2^k * vector_modulo([41/17, 13/17*sqrt(2) ], k)
  + sqrt(2)^k * vector_modulo([-16/5, 1/5], k)
  + vector_modulo([0, -3/sqrt(2) ], k)
  ;
}
check_exact(vector(12,k, nearly_equal(GRtotal(k-1),GRtotal_by_midpoints(k-1))), \
            vector(12,k, 1))

GRtotal(k) =
{
  imag ( 1/sqrt(2)^k
         * (b_real(k) - b_imag(k)*I) * (21/85 + 67/85*I)
       )
  + 2^k * vector_modulo([41/17, 30/17*sqrt(2) ], k)
  + sqrt(2)^k * vector_modulo([-16/5, -33/10], k)
  + vector_modulo([0, 0 ], k)
  ;
}
check_exact(vector(12,k, nearly_equal(GRtotal(k-1),GRtotal_by_midpoints(k-1))), \
            vector(12,k, 1))

GRtotal(k) =
{
  imag ( 1/sqrt(2)^k
         * sqrt(2)^k * if(k%2==0,1,1/2*sqrt(2))
         * (b_real_sign(k) - b_imag_sign(k)*I) * (21/85 + 67/85*I)
       )
  + 2^k * vector_modulo([41/17, 30/17*sqrt(2) ], k)
  + sqrt(2)^k * vector_modulo([-16/5, -33/10], k)
  + vector_modulo([0, 0 ], k)
  ;
}
check_exact(vector(12,k, nearly_equal(GRtotal(k-1),GRtotal_by_midpoints(k-1))), \
            vector(12,k, 1))

GRtotal(k) =
{
  imag ( 1/sqrt(2)^k
         * sqrt(2)^k * if(k%2==0,1,1/2*sqrt(2))
         * ((b_real_sign(k)) * (21/85)
         + (b_real_sign(k)) * ( 67/85*I)
         + ( - b_imag_sign(k)*I) * ( + 67/85*I)
         + ( - b_imag_sign(k)*I) * (21/85))
       )
  + 2^k * vector_modulo([41/17, 30/17*sqrt(2) ], k)
  + sqrt(2)^k * vector_modulo([-16/5, -33/10], k)
  + vector_modulo([0, 0 ], k)
  ;
}
check_exact(vector(12,k, nearly_equal(GRtotal(k-1),GRtotal_by_midpoints(k-1))), \
            vector(12,k, 1))


GRtotal(k) =
{
    if(k%2==0,1,1/2*sqrt(2))
         * (  vector_modulo([1,1,0,-1, -1,-1,0,1],k) * ( 67/85)
             - vector_modulo([0,1,1,1, 0,-1,-1,-1],k) * (21/85))
  + 2^k * vector_modulo([41/17, 30/17*sqrt(2) ], k)
  + sqrt(2)^k * vector_modulo([-16/5, -33/10], k)
  + vector_modulo([0, 0 ], k)
  ;
}
check_exact(vector(12,k, nearly_equal(GRtotal(k-1),GRtotal_by_midpoints(k-1))), \
            vector(12,k, 1))


GRtotal(k) =
{
  if(k%2==0,1,1/2*sqrt(2))
  * vector_modulo([67/85, 46/85, -21/85, -88/85, -67/85, -46/85, 21/85, 88/85],k)
  + 2^k * vector_modulo([41/17, 30/17*sqrt(2) ], k)
  + sqrt(2)^k * vector_modulo([-16/5, -33/10], k)
  + vector_modulo([0, 0 ], k)
  ;
}
check_exact(vector(12,k, nearly_equal(GRtotal(k-1),GRtotal_by_midpoints(k-1))), \
            vector(12,k, 1))


GRtotal(k) =
{
  + 2^k * vector_modulo([41/17, 30/17*sqrt(2) ], k)
  + sqrt(2)^k * vector_modulo([-16/5, -33/10], k)
  + vector_modulo([ 67/85,  23/85*sqrt(2),
                   -21/85, -44/85*sqrt(2),
                   -67/85, -23/85*sqrt(2),
                    21/85,  44/85*sqrt(2)], k)
  ;
}
check_exact(vector(12,k, nearly_equal(GRtotal(k-1),GRtotal_by_midpoints(k-1))), \
            vector(12,k, 1))

GRtotal(k) =
{
  vector_modulo([41/17       * 2^k  -16/5 * sqrt2^k,
                 30/17*sqrt2 * 2^k  -33/10 * sqrt2^k], k)
  + vector_modulo([ 67/85,  23/85*sqrt2,
                   -21/85, -44/85*sqrt2,
                   -67/85, -23/85*sqrt2,
                    21/85,  44/85*sqrt2], k)
  ;
}
check_exact(vector(12,k, GRtotal(k-1)), vector(12,k, GRtotal_by_midpoints(k-1)))

GRtotal(k) =
{
  my(h=floor(k/2));
  vector_modulo([41/17 * 2^k
                 - 16/5 * 2^h
                 + vector_modulo([ 67, -21, -67, 21], h) /85
                 ,
                 sqrt2*
                 (30/17 * 2^k
                  - 33/10 * 2^h
                  + vector_modulo([  23, -44, -23, 44], h) / 85
                 )],
                k);
}
check_exact(vector(12,k, GRtotal(k-1)), vector(12,k, GRtotal_by_midpoints(k-1)))

GRtotal(k) =
{
    1/17 * vector_modulo([41, 30*sqrt2], k) * 2^k
  - 1/10 * vector_modulo([32, 33], k) * sqrt2^k
  + 1/85 * vector_modulo([67, 23*sqrt2, -21, -44*sqrt2,
                          -67, -23*sqrt2, 21, 44*sqrt2],k)
}
check_exact(vector(12,k, GRtotal(k-1)), vector(12,k, GRtotal_by_midpoints(k-1)))

\\-----------------------------------------------------------------------------

R_by_sqrt2(k) = if(k%2,10/sqrt2,7)*sqrt2^k - 2*k - 6;
check_exact(vector(100,k, R(k-1)), vector(100,k, R_by_sqrt2(k-1)))

R_by_sqrt2(k) = vector_modulo([7, 10/sqrt2],k)*sqrt2^k - 2*k - 6;
check_exact(vector(100,k, R(k-1)), vector(100,k, R_by_sqrt2(k-1)))

GR(k) =
{
  (vector_modulo([41/17       * 2^k  -16/5 * sqrt2^k,
                 30/17*sqrt2 * 2^k  -33/10 * sqrt2^k], k)
  + vector_modulo([ 67/85,  23/85*sqrt2,
                   -21/85, -44/85*sqrt2,
                   -67/85, -23/85*sqrt2,
                    21/85,  44/85*sqrt2], k)
  ) / R_by_sqrt2(k);
}
check_exact(vector(12,k, GR(k-1)), vector(12,k, GR_by_midpoints(k-1)))

GR(k) =
{
  (vector_modulo([41/17       * 2^k  -16/5 * sqrt2^k,
                 30/17*sqrt2 * 2^k  -33/10 * sqrt2^k], k)
  + vector_modulo([ 67/85,  23/85*sqrt2,
                   -21/85, -44/85*sqrt2,
                   -67/85, -23/85*sqrt2,
                    21/85,  44/85*sqrt2], k)
  ) / (vector_modulo([7, 10/sqrt2],k)*sqrt2^k - 2*k - 6);
}
check_exact(vector(12,k, GR(k-1)), vector(12,k, GR_by_midpoints(k-1)))

GR(k) =
{
  vector_modulo([(41/17       * 2^k  -16/5 * sqrt2^k)
                 / (vector_modulo([7, 10/sqrt2],k)*sqrt2^k - 2*k - 6)
                 ,
                 (30/17*sqrt2 * 2^k  -33/10 * sqrt2^k)
                 / (vector_modulo([7, 10/sqrt2],k)*sqrt2^k - 2*k - 6)
                ], k)

  + (vector_modulo([ 67/85,  23/85*sqrt2,
                    -21/85, -44/85*sqrt2,
                    -67/85, -23/85*sqrt2,
                     21/85,  44/85*sqrt2], k)
  ) / (vector_modulo([7, 10/sqrt2],k)*sqrt2^k - 2*k - 6);
}
check_exact(vector(12,k, GR(k-1)), vector(12,k, GR_by_midpoints(k-1)))

GR(k) =
{
  vector_modulo([(41/17       * 2^k  -16/5 * sqrt2^k)
                 / (7*sqrt2^k - 2*k - 6)
                 ,
                 (30/17*sqrt2 * 2^k  -33/10 * sqrt2^k)
                 / (10/sqrt2*sqrt2^k - 2*k - 6)
                ], k)

  + (vector_modulo([ 67/85,  23/85*sqrt2,
                    -21/85, -44/85*sqrt2,
                    -67/85, -23/85*sqrt2,
                     21/85,  44/85*sqrt2], k)
  ) / (vector_modulo([7, 10/sqrt2],k)*sqrt2^k - 2*k - 6);
}
check_exact(vector(12,k, GR(k-1)), \
            vector(12,k, GR_by_midpoints(k-1)), \
            "GR")

GR(k) =
{
  vector_modulo([41/17*1/7*sqrt2^k
                 + (41/17       * 2^k  -16/5 * sqrt2^k
                   - 41/17*1/7*sqrt2^k * (7*sqrt2^k - 2*k - 6))
                   / (7*sqrt2^k - 2*k - 6)
                 ,
                 30/17*sqrt2*1/(10/sqrt2)*sqrt2^k
                 + (30/17*sqrt2 * 2^k  -33/10 * sqrt2^k
                    - 30/17*sqrt2*1/(10/sqrt2)*sqrt2^k * (10/sqrt2*sqrt2^k - 2*k - 6))
                   / (10/sqrt2*sqrt2^k - 2*k - 6)
                ], k)

  + (vector_modulo([ 67/85,  23/85*sqrt2,
                    -21/85, -44/85*sqrt2,
                    -67/85, -23/85*sqrt2,
                     21/85,  44/85*sqrt2], k)
  ) / (vector_modulo([7, 10/sqrt2],k)*sqrt2^k - 2*k - 6);
}
check_exact(vector(12,k, GR(k-1)), vector(12,k, GR_by_midpoints(k-1)))

GR(k) =
{
  vector_modulo([41/119 * sqrt2^k
                 + ( (82/119*k - 674/595 )  *sqrt2^k
                   )
                   / (7*sqrt2^k - 2*k - 6)
                 ,
                 6/17 * sqrt2^k
                 + ( (12/17*k - 201/170 ) * sqrt2^k
                   )
                   / (10/sqrt2*sqrt2^k - 2*k - 6)
                ], k)

  + (vector_modulo([ 67/85,  23/85*sqrt2,
                    -21/85, -44/85*sqrt2,
                    -67/85, -23/85*sqrt2,
                     21/85,  44/85*sqrt2], k)
  ) / (vector_modulo([7, 10/sqrt2],k)*sqrt2^k - 2*k - 6);
}
check_exact(vector(12,k, GR(k-1)), vector(12,k, GR_by_midpoints(k-1)))


GR(k) =
{
  my(h=floor(k/2));
  vector_modulo([41/119 * sqrt2^k
                 + 82/833 * k - 674/4165
                 + (164/833*k^2 + 1112/4165*k - 4044/4165
                    + vector_modulo([ 67/85, -21/85, -67/85, 21/85], h))
                   / R(k)
                 ,
                 42/119 * sqrt2^k
                 + sqrt2*(6/85*k - 201/1700 )
                 + sqrt2*(12/85 * k^2 + 159/850 *k - 603/850
                          + vector_modulo([ 23/85, -44/85, -23/85, 44/85], h))
                   / R(k)
                ], k);
}
check_exact(vector(12,k, GR(k-1)), \
            vector(12,k, GR_by_midpoints(k-1)), \
            "GR second last")

GR(k) =
{
  vector_modulo([41/119 * sqrt2^k
                 + 82/833 * k - 674/4165
                 + (820 * k^2
                    + 1112 * k
                    - vector_modulo([761,761, 5073,5073, 7327,7327, 3015,3015], k))
                   / (4165 * R(k))
                 ,
                 42/119 * sqrt2^k
                 + sqrt2*(
                    6/85*k - 201/1700 
                    + (120 * k^2 
                       + 159 * k
                       - vector_modulo([373,373, 1043,1043, 833,833, 163,163], k))
                    / (850 * R(k)))
                ], k);
}
check_exact(vector(12,k, GR(k-1)), vector(12,k, GR_by_midpoints(k-1)))

GR(k) =
{
  vector_modulo([41/119,42/119],k) * sqrt2^k
  + vector_modulo([82/833, 6/85*sqrt2],k) * k
  - vector_modulo([674/4165, 201/1700*sqrt2],k)
  +
  vector_modulo
  ([
    (820 * k^2
     + 1112 * k
     - vector_modulo([761,761, 5073,5073, 7327,7327, 3015,3015], k))
    / (4165 * R(k))
    ,
    sqrt2
    * (120 * k^2 
       + 159 * k
       - vector_modulo([373,373, 1043,1043, 833,833, 163,163], k))
    / (850 * R(k))
   ], k);
}
check_exact(vector(12,k, GR(k-1)), vector(12,k, GR_by_midpoints(k-1)))

