\\ Copyright 2015 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  If not, see <http://www.gnu.org/licenses/>.

allocatemem(300 * 10^6);
default(recover,0);
default(strictargs,1);
read("configurations.gp");
read("recurrence-guess.gp");
read("../tools/configurations-gfs-generated.gp");
\\read("c-curve-defines.gp");


\\-----------------------------------------------------------------------------
\\ gfs multiply
\\ (Id-M*x)*g = initial

(matid(size) - m*x)*conf_index_to_gf~
(matid(size) - m*x)*conf_index_to_gf~ == initial
quit();


\\-----------------------------------------------------------------------------
\\ denominator degrees which occur other than poly8(x) etc

{
  my(polys = [poly8(x),poly8alt(x),poly9(x),poly11(x),poly14(x),poly18(x)]);
  polys = apply(p->Pol(Vecrev(p)), polys);

  my(count = vector(200));
  for(i=1,#conf_index_to_gf,
     my(den = denominator(conf_index_to_gf[i]));
     for(j=1,#polys,
        while(den % polys[j] == 0, den /= polys[j]));
     
     my(f = factor(den));
     for(j=1,matsize(f)[1],
        my(deg = poldegree(f[j,1]));
        count[deg+1]++);

     \\ if(deg > 80,
     \\   print(factor(den));
     \\   quit();
     \\   );
     );

  for(i=1,#count,
     if(count[i],
        print(i-1" count "count[i])));
  quit();
}


\\-----------------------------------------------------------------------------
\\ configurations with denominator degree == 1

{
  for(index=1,#conf_index_to_gf,
     my(gf = conf_index_to_gf[index],
        deg = poldegree(denominator(gf)));
     if(deg != 1, next());

     my(conf = index_to_conf[index]);
     if(vecsum(conf) == 1, next());
     print(gf);
     conf_print(conf);
     print());
  quit();
}

\\ 8+8+9+11+14+18 == 68

\\-----------------------------------------------------------------------------
\\ denominator degrees which occur

{
  my(count = vector(200));
  for(i=1,#conf_index_to_gf,
     my(deg = poldegree(denominator(conf_index_to_gf[i])));
     if(conf_index_to_gf[i],
        count[deg+1]++));
  for(i=1,#count,
     if(count[i],
        print(i-1" count "count[i])));
  quit();
}

\\ 8+8+9+11+14+18 == 68

\\-----------------------------------------------------------------------------
\\ how many denominators with each poly

{
  my(v = [poly8(x),poly8alt(x),poly9(x),poly11(x),poly14(x),poly18(x)]);
  \\ print(v);
  for(i=1,#v,
     my(p = Pol(Vecrev(v[i])),
        count=0);
     for(i=1,#conf_index_to_gf,
        if(denominator(conf_index_to_gf[i]) % p == 0,
           count++));
     print(count" containing "p);
  );
  quit();
}

\\-----------------------------------------------------------------------------
\\ gf of initial 1 to 15

{
  for(i=1,15,
     my(conf = vector(15,j,i==j),
        index = conf_to_index(conf),
        gf = conf_index_to_gf[index]);
     print("i="i" index="index"  "gf);
    );
}
quit();


\\-----------------------------------------------------------------------------
\\ matsolve for gfs

read("../tools/configurations-gfs-generated.gp");

\\ M*vi = vi+1
\\ M*g*x = g + initial
\\ (Id-M*x)*g = initial
\\ (Id-M*x)* (g*den) = initial*den

matrix_row_swap(m, i,j) =
{
  if(i!=j,
     my(tmp = m[i,]);
     m[i,] = m[j,];
     m[j,] = tmp);
  m;
}
vector_elem_swap(v, i,j) =
{
  if(i!=j,
     my(tmp = v[i]);
     v[i] = v[j];
     v[j] = tmp);
  v;
}
\\ matrix m, column vector v
matsolve_progressively(m,v) =
{
  my(size=matsize(m)[1],
     den = 1);
  for(c=1,size,
     print("column ",c);
     for(r=c,size,
        if(m[r,c],
           m = matrix_row_swap(m,r,c);
           v = vector_elem_swap(v,r,c);
           break()));
     for(r=c+1,size,
        if(m[r,c],
           my(f = m[r,c] / m[c,c]);
           m[r,] -= m[c,] * f;
           v[r] -= v[c] * f;
           m[r,c]==0 || error();

           my(d = denominator(v[r]));
           if(d != 1,
              den *= d;
              v *= d;
              print("  denominator degree ",poldegree(den)));
          ));
  );     
}

print("matsolve");
{
  den = lcm(apply(denominator,conf_index_to_gf))
       * (-2*x^2 + 1)
       * (-x^3 - x^2 + 1) * (-x^3 + 1)
       * (x^5 + x^4 + x - 1) * (2*x^3 - 2*x^2 + 2*x - 1) * (-x^8 - 2*x^2 + 1)
       * (-3*x^6 + 3*x^4 - 3*x^2 + 1)
       * (-6*x^16 - 6*x^14 + 3*x^13 - 3*x^12 - x^11 - 3*x^10 - 5*x^9 + 7*x^8 - 2*x^7 - 2*x^6 + x^5 + x^4 - 2*x^3 + 2*x^2 + x - 1)
       * (-6*x^16 - 8*x^14 - x^12 - x^11 - 6*x^10 - 2*x^9 + 7*x^8 - 2*x^7 - 2*x^6 + x^5 + x^4 - 2*x^3 + 2*x^2 + x - 1)
       * (-6*x^16 - 7*x^14 - x^12 - 2*x^11 - 5*x^10 - 2*x^9 + 7*x^8 - 2*x^7 - 2*x^6 + x^5 + x^4 - 2*x^3 + 2*x^2 + x - 1)
       * (-6*x^21 - x^19 + 6*x^18 - x^17 + 2*x^16 - 2*x^15 - 8*x^14 + 13*x^13 + 5*x^12 - 7*x^11 - 9*x^10 + 3*x^9 + 6*x^8 - 2*x^7 - 2*x^4 - 2*x^3 + 3*x^2 + x - 1)
       ;
}
den = 1;
print("den ",den);
mx = matid(size) - m*x;
matsolve_progressively(mx, initial*den);
gm = matsolve(mx, initial*den);
gm
quit();

