\\ Copyright 2015, 2016, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  If not, see <http://www.gnu.org/licenses/>.

default(recover,0);
read("c-curve-defines.gp");

check_count = 0;
check_exact(got,want, name="") =
{
  check_count++;
  if(got!= want,
     print("\nwrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

PartN(k) = 1/3*(2^k + vector_modulo([ -1,1] ,k));
PartSegments(k) = PartN(k+1) - PartN(k);
CountTrailingZeros(n) = valuation(n,2);
check_exact(CountTrailingZeros(1), 0);
check_exact(CountTrailingZeros(8), 3);
check_exact(CountTrailingZeros(12), 2);

\\-----------------------------------------------------------------------------

PartDiameter(k) = [1,0,2,2,4,6,9,15,19,31,43,67,93,137,187,275,381][k+1];

\\ *===* 2
\\
\\    *
\\  /   \  4
\\ *     *
\\  \   /
\\    *
\\
\\ *---*---*
\\ |       |  8
\\ *       *
\\ |       |
\\ *---*---*

OctagonB(k) = if(k==0,2, 4*(Rpart(k) + Rpart(k-1)));
check_exact(OctagonB(1), 4);
check_exact(OctagonB(2), 8);
check_exact(OctagonB(3), 16);
check_exact(OctagonB(4), 4*4+4*2);

TailLength(k) = sum(i=0,k, Rpart(i));
check_exact(TailLength(0), 1, "TailLength(0)");
check_exact(TailLength(1), 1);
check_exact(TailLength(2), 3);
check_exact(TailLength(3), 5);
check_exact(TailLength(4), 9, "TailLength(4)");

oct_descent(k) = \
  my(total = 0); \
  while(k >= 0, total += R(k); k-=2); \
  total;

check_exact(oct_descent(0), 1);
check_exact(oct_descent(1), 2);
check_exact(oct_descent(2), 1 + 4);

PartDiameterBound(k) =
{
  if(k<=4, return(Rpart(k)));
  vecmax([
          1/2*Rpart(k) + oct_descent(k),

          \\ R(k-2),
          \\ R(k-2) + R(k-4),
          \\ R(k-2) + R(k-2),

          \\ R(k-2),
          \\ Rpart(k) + R(k-2),
          \\ R(k-2),          \\ first 2
          \\ R(k-2) + R(k-3), \\ middle double
          \\ R(k-2) + R(k-2), \\ end 2
          0
         ]);
  \\ R(k-2) + R(k-3);
  \\ R(k-2) + R(k-2);
  
  \\ my(ret = 0);
  \\ forstep(i=2, Rpart(k)-2, 2,
  \\        my(this = oct_descent(CountTrailingZeros(k)));
  \\        if(this > ret, ret = this);
  \\ );
  \\ ret;

  \\ oct_descent(k-4) + oct_descent(k-6) + Rpart(k);

  2^floor(k/2) + R(k-4) + Rpart(k);
}


print("actual ",vector(16,k,k--; PartDiameter(k)));
print("tail   ",vector(18,k,k--; TailLength(k)));
print("bound  ",vector(18,k,k--; PartDiameterBound(k)));
vector(18,k,k--; TailLength(k) > PartDiameterBound(k))
\\ vector(18,k,k--; PartSegments(k))