#!/usr/bin/perl -w

# Copyright 2015, 2016, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::CCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyCCurve 'mul','div','xy_rotate_minus90','chamfer',
  '@dir4_to_dx', '@dir4_to_dy';
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 11;

print "  % generated by tools/$FindBin::Script\n";
print "  \\begin{scope}[shift=(pos-QI)]\n";
my $path = Math::PlanePath::CCurve->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

# find QI4(11)=1 first 4-inward square
my ($qi4_x,$qi4_y);
my @qi4_n_list;
N: foreach my $n ($n_lo .. $n_hi-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  my ($rx,$ry) = xy_rotate_minus90($dx,$dy);

  my ($x2,$y2) = ($x+$dx, $y+$dy);
  my ($x3,$y3) = ($x2+$rx, $y2+$ry);
  my ($x4,$y4) = ($x +$rx, $y +$ry);

  my @n_list = ($path->xyxy_to_n($x2,$y2, $x3,$y3),
                $path->xyxy_to_n($x3,$y3, $x4,$y4),
                $path->xyxy_to_n($x4,$y4, $x, $y));
  foreach my $other_n (@n_list) {
    (defined $other_n && $other_n < $n_hi)
      or next N;
  }
  ($qi4_x,$qi4_y) = ($x,$y);
  @qi4_n_list = sort {$a<=>$b} $n, @n_list;
  last;
}
my ($qi4_s,$qi4_d) = mul($qi4_x, $qi4_y);
print "  % QI4 at N=",join(',',@qi4_n_list)," at $qi4_x,$qi4_y, sd=$qi4_s,$qi4_d\n";

my $qi4_s_min = $qi4_s - 1;
my $qi4_s_max = $qi4_s + 3;
my $qi4_d_min = $qi4_d - 2;
my $qi4_d_max = $qi4_d + 2;

sub in_range {
  my ($s1,$d1, $s2,$d2) = @_;
  return ($qi4_s_min <= $s1 && $s1 <= $qi4_s_max
          && $qi4_d_min <= $d1 && $d1 <= $qi4_d_max
          && $qi4_s_min <= $s2 && $s2 <= $qi4_s_max
          && $qi4_d_min <= $d2 && $d2 <= $qi4_d_max

          && ! ($s1 == $s2 && ($s1 == $qi4_s_min || $s1 == $qi4_s_max))
          && ! ($d1 == $d2 && ($d1 == $qi4_d_min || $d1 == $qi4_d_max))
         );
}


#                     *  x2,y2
#                p  / |
#              a  /   |
#            g  /     |g|
#             /       |a|
#           /         |p|
#         /           |
# x1,y1 *-------------*  x3,y3
#           | gap |
sub triangle {
  my ($x1,$y1, $x2,$y2) = @_;
  ($x1,$y1) = mul($x1,$y1);
  ($x2,$y2) = mul($x2,$y2);
  my ($dx,$dy) = ($x2-$x1, $y2-$y1);
  my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
  my ($x3,$y3) = ($x1+$dx/2+$rx/2, $y1+$dy/2+$ry/2);

  my $x1delta = 0;
  my $x2delta = 0;
  my $x3delta = 0;
  my $y1delta = 0;
  my $y2delta = 0;
  my $y3delta = 0;

  if (in_range($x1,$y1, $x2,$y2)) {
    # printwrap(" \\pic at ($x1,$y1) [transform shape] {my triangle={$dx,$dy}};");

    # lower and right gaps
    $x1delta += $dx;  # x1,y1 diagonally up
    $y1delta += $dy;
    $x2delta += -$dx;  # x2,y2 diagonally down
    $y2delta += -$dy;

    $y3delta += -$ry;
    $x3delta += -$rx;

    # no back-to-back triangles, no need for diagonal gap
    # $x1delta += $dx;  # diagonal gap
    # $y2delta += -$dx;

    $x1 .= chamfer($x1delta,'\\MyGap');
    $x2 .= chamfer($x2delta,'\\MyGap');
    $x3 .= chamfer($x3delta,'\\MyGap');
    $y1 .= chamfer($y1delta,'\\MyGap');
    $y2 .= chamfer($y2delta,'\\MyGap');
    $y3 .= chamfer($y3delta,'\\MyGap');

    printwrap(" ($x1,$y1)--($x2,$y2)--($x3,$y3)--cycle");
  }
}

$Printwrap = 2;
printwrap("\\fill [my grey]");
foreach my $s ($qi4_s_min .. $qi4_s_max) {
  foreach my $d ($qi4_d_min .. $qi4_d_max) {
    next if ($s+$d)&1;
    my ($x,$y) = div($s,$d);
    $x /= 2; $y /= 2;
    ### point: "sd=$s,$d  xy=$x,$y"

    foreach my $dir4 (0 .. 3) {
      my $dx = $dir4_to_dx[$dir4];
      my $dy = $dir4_to_dy[$dir4];

      my ($x2,$y2) = ($x+$dx, $y+$dy);
      my $n = $path->xyxy_to_n($x,$y, $x2,$y2);
      next unless defined $n && $n < $n_hi;

      triangle($x,$y, $x2,$y2);
    }
  }
}
print ";";
printwrap("\n");

print "  \\coordinate (start) at (0,0);\n";
{
  my ($x,$y) = mul($x_hi,$y_hi);
  print "  \\coordinate (end) at ($x,$y);\n";
}

{
  my $x = ($qi4_s_min + $qi4_s_max) / 2;
  my $y = $qi4_d_min;
  print "  \\coordinate (QI4-minimum) at ($x,$y);\n";
}
print "  \\end{scope}\n";

#------------------------------------------------------------

print "  \\begin{scope}[shift=(pos-V),shorten <=\\MyShortenStart,shorten >=\\MyShortenEnd]\n";
($n_lo, $n_hi) = $path->level_to_n_range($level+1);

$Printwrap = 2;
foreach my $x ($qi4_s_min .. $qi4_s_max) {
  foreach my $y ($qi4_d_min .. $qi4_d_max) {

    foreach my $dir4 (0 .. 3) {
      my $dx = $dir4_to_dx[$dir4];
      my $dy = $dir4_to_dy[$dir4];

      my ($x2,$y2) = ($x+$dx, $y+$dy);
      if (in_range($x,$y, $x2,$y2)) {

        my $n = $path->xyxy_to_n($x,$y, $x2,$y2);
        next unless defined $n && $n < $n_hi;

        my @n_list = $path->xyxy_to_n_list_either($x,$y, $x2,$y2);
        @n_list = grep {$_<$n_hi} @n_list;
        next unless @n_list;

        my $dx = $x2 - $x;
        my $dy = $y2 - $y;
        ($dx,$dy) = xy_rotate_minus90($dx,$dy);
        my $sx = $x . chamfer($dx,'\MyOverlapGap');
        my $sy = $y . chamfer($dy,'\MyOverlapGap');
        my $tx = $x2 . chamfer($dx,'\MyOverlapGap');
        my $ty = $y2 . chamfer($dy,'\MyOverlapGap');
        printwrap(" \\draw[my harpoon] ($sx,$sy)--($tx,$ty);");

        # if (@n_list == 1) {
        #   printwrap(" \\draw[->] ($x,$y)--($x2,$y2);");
        # } elsif (@n_list == 2) {
        # }
      }
    }
  }
}
printwrap_newline();
{
  my $x = ($qi4_s_min + $qi4_s_max) / 2;
  my $y = $qi4_d_min;
  print "  \\coordinate (V4-minimum) at ($x,$y);\n";
}
print "  \\end{scope}\n";

print "  % end generated\n";
exit 0;
