#!/usr/bin/perl -w

# Copyright 2014, 2015, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use List::Util 'min','max';
use Math::BigRat;
use Math::PlanePath::CCurve 117; # v.117 for level_to_n_range()
use Math::Geometry::Planar ();

use FindBin;
use lib $FindBin::Bin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyCCurve 'xydxdy_triangle_neighbours','PartN';
$| = 1;

# uncomment this to run the ### lines
# use Smart::Comments;


my $max_level = 10;
if (@ARGV) { $max_level = shift @ARGV; }

print "% generated by tools/$FindBin::Script $max_level\n";
print "% GP-DEFINE  Trigons_generated_samples = [";

my $path = Math::PlanePath::CCurve->new;
foreach my $k (0 .. $max_level) {
  my ($n_lo, $n_hi) = $path->level_to_n_range($k);
  # my $n_lo = PartN($k);
  # my $n_hi = PartN($k+1);
  ### at: "k=$k N=$n_lo to $n_hi" 
  my $count_trigons = 0;
  my $seen = '';  # bit vector indexed by $n

  foreach my $n ($n_lo .. $n_hi-1) {
    next if vec($seen,$n,1);

    # this triangle is a new polygon piece, extend out its neighbours
    $count_trigons++;
    vec($seen,$n,1) = 1;
    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);
    my @pending = (xydxdy_triangle_neighbours($x,$y, $dx,$dy));
    while (my $elem = shift @pending) {
      my ($x,$y, $dx,$dy) = @$elem;
      my $n = $path->xyxy_to_n ($x,$y, $x+$dx,$y+$dy) // next;
      $n < $n_hi or next;
      next if vec($seen,$n,1);
      vec($seen,$n,1) = 1;
      push @pending, xydxdy_triangle_neighbours($x,$y, $dx,$dy);
    }
  }
  print $count_trigons, $k < $max_level ? ',' : '';
}
print "];\n";
print "% end generated\n";
exit 0;
