#!/usr/bin/perl -w

# Copyright 2013, 2014, 2015, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  See the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.


# Usage: perl test.pl
#
# Run some self-tests on ../c-curve.tex and on MyCCurve.pm.
#

use 5.010;
use strict;
use File::Slurp;
use IPC::Run;
use List::Util 'max','min';

use FindBin;
use lib::abs $FindBin::Bin;
use MyCCurve ':all';


my $source_dir = File::Spec->catdir ($FindBin::Bin, File::Spec->updir);
$source_dir = File::Spec->rel2abs($source_dir);
print "source dir $source_dir\n";

chdir $source_dir or die $!;

my $c_curve_tex_filename = 'c-curve.tex';
# $c_curve_tex_filename = 'devel/more.tex';
my $c_curve_pdf_filename = 'c-curve.pdf';
my $c_curve_tex_str = File::Slurp::slurp($c_curve_tex_filename);
print count_lines($c_curve_tex_str)," lines\n";

my $c_curve_tex_str_sans_comments = $c_curve_tex_str;
$c_curve_tex_str_sans_comments =~ s/((^|[^\\])(\\\\)*)%[^\n]*/$1/mg;
print count_lines($c_curve_tex_str_sans_comments)," lines\n";


#------------------------------------------------------------------------------
# ../devel/c-curve-defines.gp
# ../devel/c-curve-defines.header.gp

{
  # same copyright years as c-curve.tex where defines are extracted from
  $c_curve_tex_str =~ /^% (Copyright .*)/m or die;
  my $want_copyright = $1;
  foreach my $filename ('devel/c-curve-defines.gp',
                        'devel/c-curve-defines.header.gp') {
    my $str = File::Slurp::slurp($filename);
    $str =~ /^\\\\ \Q$want_copyright\E$/m
      or die "$filename copyright different from c-curve.tex";
  }
}

#------------------------------------------------------------------------------
# c-curve.pdf Title and Author

{

  my $pdfinfo_out;
  IPC::Run::run(['pdfinfo', '-isodates', $c_curve_pdf_filename],
                '>', \$pdfinfo_out)
      or die;
  $pdfinfo_out =~ s/^Jav.*\n//m;
  print $pdfinfo_out;
  {
    $pdfinfo_out =~ /^Title:\s*(.*)/m
      or die;
    my $title = $1;
    # print "title: $title\n";
    $title =~ /C Curve/
      or die;
  }
  {
    $pdfinfo_out =~ /^Author:\s*(.*)/m
      or die;
    my $author = $1;
    # print "author: $author\n";
    $author =~ /Kevin/
      or die;
  }
}

#------------------------------------------------------------------------------
sub pos_linenum {
  my ($str) = @_;
  $str = substr($str, 0, pos($_[0]));
  return 1 + scalar($str =~ tr/\n//);
}

sub count_lines {
  my ($str) = @_;
  my $count = ($str =~ tr/\n/\n/);
  if ($str =~ /[^\n]$/) { $count++; }
  return $count;
}

#------------------------------------------------------------------------------
exit 0;
